pro read_trace, files, dsets, index, data, index0=index0, $
	nodata=nodata, qdebug=qdebug, qstop=qstop, $
	db_file=db_file, taglist=taglist, tagfile=tagfile, all_tags=all_tags, $
	alphabetize=alphabetize, catalog=catalog, $
	rotate=rotate, loud=loud, outsize=outsize, mag_factor=mag_factor, $
	integer=integer, dtype=dtype, original=original, $
	fileinfo=fileinfo, phead=phead
;+
;   Name:  read_trace
;
;   Purpose: Read TRACE data image files, headers or headers & data
;
;   Input Parameters:
;      files - list of one or more files to read
;      dsets - integer indices
;
;   Output Parameters:
;     index - vector of IDL structures with desired tags decoded
;     data  - 2D/3D data (one image per dset request)
;  
;   Calling Sequence:
;      read_trace, files, dsets, index, data
;      read_trace, infil, [4,5,6], index, data, /nodata
;
;   Keyword Parameters:
;      taglist     - optional list of tags to include (pass to trace_dph2struct)
;      alphabetize - switch, if set, structure tags are alpha (def=db order)
;      catalog     - if set, taglist is CATALOG subset
;      all         - if set, taglist is all in keyword dbase
;      fileinfo    - if set, include some fields from PRIMARY header
;                    (switch for defaults or comma delimited set of PH tags)
;      loud        - if set, show some status
;      original -  if set, force DPH decode (override FILE_TYPE)
;      phead (Output) - vector of primary header structures (1 per file)
;      outsize/mag_factor - rebin the images appropriately
;
;HISTORY:
;	Written 15-Jan-98 by M.Morrison via call to S.L.Freeland routines
;                          mxf_xxx and trace_dph2struct
;	30-Jan-98 (MDM) - Added "dph" to the arguments passed to dph2struct
;        2-Mar-98 (SLF) - add TAGFILE keyword and function
;                         made no data output parameter imply /NODATA
;        4-Mar-98 (SLF) - add ALPHABETIZE keyword (pass to trace_dph2struct)
;       10-Mar-98 (SLF) - add /CATALOG keyword (map via keywords_catalog.tdb)
;       11-Mar-98 (SLF) - add /ROTATE keyword (pass to mxf_read_data)
;                         add /MAG_FACTOR and OUTSIZE (pass to mxf_read_data)
;       15-Mar-98 (SLF) - add DATATYPE (careful - override value in mxf structure)
;                         and /INTEGER (DATATYPE=>2)
;       19-Mar-98 (SLF) - pass /LOUD -> trace_dph2struct
;       23-Mar-98 (SLF) - add FILE_TYPE check, add /ORIGINAL
;       25-Mar-98 (SLF) - change file_type from extension header to primary
;                         return PHEAD, add FILEINFO keyword and function
;       24-apr-98 (SLF) - pointing info->SSW
;       27-Apr-98 (SLF) - fix TAGFILE bug
;        9-May-98 (SLF) - permit CRVAL w/o CRPIX (catalog, for example)
;       11-Jun-98 (SLF) - permit flat FITS file reading (just call mreadfits)
;-         
loud=keyword_set(loud)
if keyword_set(integer) then dtype=2     ; override incorrect data type
if (n_elements(db_file) eq 0) then db_file = concat_dir('$TR_CAL_INFO', 'trace_keywords.tdb')

case 1 of
    (keyword_set(all_tags)):  delvarx, taglist	;all tags in db_file
    keyword_set(catalog): tagfile = concat_dir('$TR_CAL_INFO', 'keywords_catalog.tdb')
    data_chk(tagfile,/string): 
    else: if (n_elements(taglist) eq 0) then $
	  tagfile = concat_dir('$TR_CAL_INFO', 'keywords_image1.tdb')
endcase

nodata=keyword_set(nodata) or n_params() lt 4 ; why read if no ouput?? S.L.F.
;
flatfits=total(  (strpos(files,'.fits') ne -1) or $
	         (strpos(files,'.fts')  ne -1)  ) gt 0

if flatfits then begin 
   if loud then box_message,'Flat Fits assumed, calling mreadfits'
   case 1 of 
      n_elements(dsets) eq 0: ss=lindgen(n_elements(files))
      dsets(0) eq -1: ss=lindgen(n_elements(files))
      else: ss=dsets
   endcase      
   mreadfits,files(ss),index,data, outsize=outsize
   return                                      ;<<<<<<<<<<< EARLY EXIT
endif

if nodata then begin
    ;---- Temporary work-around
    mxf_read_header, files, index0, map, /ext, phead=phead
    if (data_type(dsets) eq 8) then begin	;passed in a structure
	vmap = map.dset + map.ifile/float(max(map.ifile))
	vdset= dsets.dset + dsets.ifile/float(max(map.ifile))
	ss = where_arr(vmap, vdset)
	index0 = index0(ss)
    end else begin
	if (dsets(0) ne -1) then index0 = index0(dsets)
    end
end else begin
    mxf_read_data, files, dsets, index0, data, $
		   rotate=rotate, phead=phead, $
		   loud=loud, outsize=outsize, mag_factor=mag_factor, dtype=dtype
end
;;if (keyword_set(nodata)) then mxf_read_header, files, dsets, index0, /ext $
;;			 else mxf_read_data,   files, dsets, index0, data, rotate=rotate, loud=loud

; S.L.F. 25-march-1998 - determine the file_type from Primary header
file_type=max(gt_tagval(phead,/filetype,found=ftfound)) > 0

original=keyword_set(original) 
ftype='File Type ' + strtrim(file_type,2)
mess=([ftype,ftype+',... /ORIGINAL set'])(original and file_type>0)

; -------------------------------------------------------------
if file_type eq 0 or (1-ftfound) or original then begin
  if loud then box_message,mess + ' ,... decoding DPH'
  index = trace_dph2struct(ph, index0, dph, loud=loud, $
        taglist=taglist, tagfile=tagfile, head_db=db_file, alphabetize=alphabetize)
endif else begin
  if loud then box_message,ftype+', ...using saved (not DPH) '
  index=index0   
endelse
; -------------------------------------------------------------

; --------- include some primary header info on request -----------
if keyword_set(fileinfo) then begin
   if loud then box_message,'Adding File Info'
   if data_chk(fileinfo,/string) then ptags=fileinfo else $
         ptags='PROG_NAM,PROG_VER,PROG_RUN,COMMENT'    ; default
   dsetmap=mxf_dset_map(phead,dsets)                   ; make map
   substr=str_subset(phead(dsetmap.ifile),ptags)       ; ->pH subset
   fnames=files(dsetmap.ifile)                         ; filenames
   fnames=strmids(fnames, str_lastpos(fnames,get_delim())+1,strlen(fnames))
   if n_elements(fnames) eq 1 then fnames=fnames(0)    ; RSI work around
   substr=add_tag(substr,'','FILENAME',index=-1) & substr.FILENAME=fnames
   index=join_struct(index,substr)      
endif
; -------------------------------------------------------------

if required_tags(index,'xcen,ycen,crval1,crval2') then begin
   if total([gt_tagval(index,/crval1,miss=0), $
             gt_tagval(index,/crpix1,miss=0)]) eq 0 then begin
    if required_tags(index,'crpix1,crpix2') then begin
        index.crpix1=1
	index.crpix2=1
    endif
   index.crval1=trace_cen2pix(index,/xpix)
   index.crval2=trace_cen2pix(index,/ypix)
   endif  
endif  

if (keyword_set(qstop)) then stop
end
