function trace_file2path, inputs, online, times=times
;
;+   Name: trace_file2path
;
;    Purpose: derive full pathname to file
;
;    Output:
;       function returns full path to input files
;  
;    Input Parameters:
;      inputs - list of files, with or without paths
;
;    Output Parameters:
;      online - boolean, =1 if files actually online
;  
;    Calling Sequence:
;       filenames=trace_file2path(filelist [,online] )
;  
;    Calling Examples:
;       IDL> print,trace_file2path('tri19980515.0000',online)
;
;       hosts/vestige/archive/sci/trace/tri/week19980510/tri19980515.0000  
;       | -------- $TRACE_I1_DIR ---------|-- weekid ---|--- filename ---|
;  
;       IDL> print,online    ; actually online locally?
;          1                 ; yes in this case
;  
;    Method:
;       check current, if not found, construct standard path inc, 'week...'
;    
;    Restrictions:
;      Unless files in current directory, assume $TRACE_I1_DIR is parent
;      In a given call, cannot have mixed locations
;      (for ex: PWD and $TRACE_I1_DIR/week.../xxx)
;
;   History:
;      21-May-1998 - S.L.Freeland  
;
;   Calls:
;      get_logenv, curdir, file2time, anytim2weekinfo, concat_dir
;-

topdir=get_logenv('TRACE_I1_DIR')     ; usually the top level

if topdir(0) eq '' then begin
   message,/info,"$TRACE_I1_DIR not defined, using current directory"
   topdir=curdir()
endif

if not keyword_set(times) then times=file2time(inputs,/ecs) 

testf=concat_dir(topdir,inputs(0))
testw=concat_dir(topdir,'week'+anytim2weekinfo(times(0),/first))

case 1 of 
   file_exist(testf): retval=inputs                          ; current dir
   file_exist(testw): retval=concat_dir(concat_dir(topdir,$
				   'week'+anytim2weekinfo(times,/first)),inputs)
   else:
endcase   

online=file_exist(retval)
if n_elements(retval) eq 0 then begin  ; scalarize if one element
  retval=retval(0)
  online=online(0)
endif

return,retval
end
