pro trace_last_movie, minframes=minframes, maxframes=maxframes, $
		      prepit=prepit,  outsize=outsize,  _extra=_extra, $
		      mdir=mdir, clean=clean
;+
;   Name: trace_last_movie
;
;   Purpose: recent TRACE movies -> WWW  
;
;   Keyword Parameters:
;      minframes - ignore movies with fewer than this many frames
;      maxframes - upper limit number of frames per movie
;      outsize   - output size for each movie frame
;      prepit  - switch, if set, run data through trace_prep
;      _EXTRA  - special_movie / image2movie keywords via inheritance
;
;   History:
;      25-April-1998 - S.L.Freeland - apply some SSW movie techniques->TRACE  
;      12-Jun-1998   - S.L.Freeland - call 'trace_data_filter' to remove
;                      radiation contaminated/other bad data  
;
;   Method:
;      use trace_cat - loop through 'special_movie' for each
;                      uniq movie dset - organize submovies by OBS_PROG
;
;   Calls:
;      trace_cat, trace_cat2data, read_trace, trace_uniq_movies 
;      special_movie (image2movie), ssw_ct2rgb, set_logenv, get_logenv
;      html_doc, file_append, the usual suspects
;
;   TODO:
;      add the FOV discrimiator (via trace_uniq_movies.pro mod)  
;-
trace_last_movie_queue                   ; move current->lm_YYMMDDD

; ============ should use configure_http,/trace ============
tophttp='/DataCenter/www/htdocs/TRACE/
if n_elements(mdir) eq 0 then mdir=concat_dir(tophttp,'last_movies') else $
    mdir =concat_dir(tophttp,mdir)
box_message,'Using movie directory: ' + mdir(0)
set_logenv,'path_http',tophttp
set_logenv,'top_http','http://vestige.lmsal.com/TRACE/' 
cd,mdir

set_plot,'Z'                                            ; use the Z buffer
; ============================================================

prepit=keyword_set(prepit)                        ; trace_prep on request

; --------- read the recent TRACE catalog -----------------
trace_cat,catalog,/last64                         ; use the catalog
if not data_chk(catalog,/struct) then begin
    box_message,'Last catalog entry > 64 hours old'
    return
endif

; --------- init some defaults ------------------
if not keyword_set(percent) then percent=100
if not keyword_set(outdize) then outsize=300
if n_elements(minframes) eq 0 then minframes=5     ; not enough, dont bother 
if n_elements(maxframes) eq 0 then maxframes=30    ; upper limit
; -------------------------------------------------
   
; ------ find 'uniq' movies within these catalog entries ---------
umovies=trace_uniq_movies(catalog,minframes=minframes, $  ; uniq movies
			  exclude='dark,????,testsnap')   ; only 'interesting'

uobs=uniq(umovies.obs_prog, sort(umovies.obs_prog))  ; number of top level WWW docs
; -------------------------------------------------

; --------- for each observing program ------------
for i=0,n_elements(uobs)-1 do begin             ; for each observing program
   uss=where(umovies.obs_prog eq umovies(uobs(i)).obs_prog,mcnt)
   thisobsprog=umovies(uobs(i)).obs_prog
   box_message,'Number of records for OBS_PROG ' + thisobsprog +' :'+strtrim(mcnt,2) 
   allhtml='<h2>'+ thisobsprog +'</h2>'            ; initialize html header

   ;  -------- for each submovie -------------
   for m=0,mcnt-1 do begin 

   ;   -------- map this movie -> SS relative to catalog ---------
      thismovie=umovies(uss(m))                          ; pointer
      ss=reverse(trace_uniq_movies(catalog,thismovie))   ; subscripts for 1 mov
      ssgood=trace_data_filter(catalog(ss),count=count)
;     --------- filter out bad frames ------------
      if count gt 0 then ss=ss(ssgood)                   ; filter out bad
      if count ne thismovie.nmatches then begin
         box_message,['Data Quality Filter', $
		      'Rejected ' + strtrim(thismovie.nmatches-count,2) + ' images']
      endif	
      if n_elements(ss) lt minframes then begin
	box_message,'Good frames dropped below MINFRAMES, aborting this movie'
      endif else begin 

;     ------------- GOOD MOVIE BLOCK -------------------------
      ss=reverse(ss(0:maxframes<(min(n_elements(ss),count)-1))) ; limit frames

;     --------- map catalog->data --------------------
      trace_cat2data,catalog(ss),files,dset,/filedset      ; catalog->filedset
      read_trace, files, dset, index, data,outsize=outsize ; read data

;     ------- correct data (trace_prep) or just apply some scale ---------
      if prepit then trace_prep,index,data,oindex,odata else begin
      nx=gt_tagval(index(0),/naxis1,missing=0)
      wl=gt_tagval(index(0),/wave_len,missing='??')
      autoscaleit=is_number(wl)
      if  autoscaleit then begin
           box_message,['Auto scaling, Wave='+wl]
	   chkscl=tracescale(data(*,*,3<(n_elements(index)-1)),/byte,/despike,returnscal=rts)
	   for j=0,n_elements(index)-1 do data(0,0,j)=tracescale(data(*,*,j), scaling=rts, /byte,/despike)
         endif else begin 
            mdata=87. & scalemax=.9*average(sqrt(index.img_max))
            for j=0,n_elements(index)-1 do data(0,0,j)=bytscl(sqrt(data(*,*,j)-mdata>0) , max=scalemax)
	    data=temporary(byte(data))
         endelse
      endelse

      trace_colors,fix(thismovie.wave_len),r,g,b   ; wavelenght

;     ---- data->WWW in 3 standard formats via image2movie)
      special_movie, index, data, r, g, b, /no_html_doc, html=html, $
          movie_name=thismovie(0).uname, _extra=_extra, $
          movie_dir=mdir, /label, outsize=outsize
      allhtml=[allhtml,"<h4>Wavelength: " + thismovie.wave_len + "&#197"+ $
		   "; Source Area: " +  strtrim(thismovie.sou_area,2)+"</h4>",html]
    endelse ; ---------- end GOOD movie block -------
    endfor  ; ---------- end of submovie loop -----------

;   ================ write the top level html documnet ==============
    hdoc=concat_dir(mdir,'LATEST_TRACE_'+thisobsprog+'.html')
    html_doc,hdoc,/header
    file_append,hdoc, str_replace(allhtml,'http://./','')
    html_doc,hdoc,/trailer
;   =====================================================

endfor     ; ------- end of OBS_PROG loop -------------------

; ----------- (re)generate the top level html index file --------
trace_movie_index

; =====================================================
return
end
