function trace_scale, index, data, outindex, $
     despike=despike,$                   ; if set, call trace_despike 
     contrast=contrast,$                 ; scaled range of intensities (def=3)
     exceed=exceed,$                     ; no of pixels saturated (def=500)
     scaling=scaling,$                   ; array prescribing scaling
     returnscaling=returnscaling,$       ; array specifying scaling 
     byte=byte, $                        ; scale to a byte image, max: byte
     fullrange=fullrange, $              ; if set, don't leave 2 cells available
     magrange=magrange,$                 ; if set, leave 2 cells (Karels default)
     loud=loud,           $              ; if set, print some diagnostics
     overwrite=overwrite, $              ; if set, ok to clobber input
;                                        ; (memory management for 3D case)
     reference_image=reference_image     ; index record or image number
;+
;   Name: trace_scale
;
;   Purpose: scale TRACE images (SSW wrapper for Karel Schryver routines)
;
;   Input Parameters:
;      index  - TRACE 'index' from read_trace or mreadfits
;      data   - data array
;     
;    History:
;      30-Apr-1998 - tracescale by C.J. Schryver Written
;      21-May-1998 - tracescale C.J. Schryver modified, 'submitted'
;      28-May-1998 - S.L.Freeland - made trace_scale wrapper, online->SSW
;                    remove file reading code, made default quiet , $
;                    added /LOUD and /FULLRANGE switches permit 3D input and 
;    Output:
;         image_scaled      despiked&scaled image - bytarr
;         image_corrected   despiked image - intarr (optional)
;
;    Method:
;       (see header for 'tracescale.pro' 
;
;    Calling Examples:
;       ; derived scaling ...
;          IDL> simg=trace_scale(index,data,/despike,/byte,contrast=3)
;       ; prescribed scaling
;          IDL> simg=trace_scale(index, data,/despike,scaling=[p,s,g])
;               (p=pedestal/minimum, s=saturation/maximum, g=gamma)
;
;      Scaling adjusts to available colors (use Zbuffer for full 256->GIF)
;      By default, the two highest cells are not used and are available
;           for magnetic contours - use /FULLRANGE to override
;-
version=1.0
loud=keyword_set(loud)
if keyword_set(contrast) then contrast=contrast else contrast=3.5
byteit=keyword_set(byte)                   ; multi-valued, set a flag
despike=keyword_set(despike)               ; boolean, set it
fullrange=1-keyword_set(magrange)          ; default is full range

if not data_chk(index,/struct) then begin
   box_message,['IDL> scaled_data=trace_scale(index, data [, etc]']
   return,-1
endif   

if keyword_set(overwrite) then outdata=temporary(data) else outdata=data
outdata=call_function((['float','fix'])(byteit),temporary(outdata))

nimgs=n_elements(index)
refimg=-1
imdone=intarr(nimgs)

; ---------- if reference image specified, use that one for 3D scaling -----
if nimgs gt 1 and n_elements(reference_image) gt 0  then begin
  if data_chk(reference_image,/struct) then $
      refimg=time2dset(index,reference_image) else refimg=reference_image
  outdata(0,0,refimg)=tracescale(data(*,*,refimg), $
     byte=byte, exceed=exceed, despike=despike, loud=loud, fullrange=fullrange, $
     returnscaling=scaling )          ; <<<< output scaling used for all images
  imdone(refimg)=1                    ; dont repeat this one
endif  
; --------------------------------------------------------------------

wanthist=n_params() eq 3

histtext ='trace_scale; Version' + string(version,format='(f5.2)')
spiketext='trace_scale; '+(['NOT Despiked','DESPIKED'])(keyword_set(despike))
scaletext='trace_scale; '

; ----------- if 3rd parameter, assume user wants history --------
if wanthist then begin 
   outindex = index                              ; track scaling history
   if tag_index(outindex,'history') eq -1 then $
       outindex=add_tag(outindex,histtext,'HISTORY') else $
       outindex=boost_tag(outindex,histtext,'HISTORY')
   outindex=boost_tag(outindex,spiketext,'HISTORY')
   outindex=boost_tag(outindex,scaletext,'HISTORY')
   hpnt=n_elements(outindex(0).HISTORY)-1
endif 

;------------ image scaling loop ---------------------
for imgn=0,nimgs-1 do begin                   ; correction scaled loop
   if not imdone(imgn) then begin
       if is_number(gt_tagval(index(imgn),/wave_len,missing='??')) then begin
          outdata(0,0,imgn)=tracescale(outdata(*,*,imgn), $
            byte=byte, exceed=exceed, despike=despike, $
            loud=loud, fullrange=fullrange, scaling=scaling, returnscaling=rts)
       endif else outdata(0,0,imgn)=$
	      bytscl(alog10(outdata(*,*,imgn)-83.>.01))           
   endif
   if wanthist then begin
     outindex(imgn).history(hpnt) =outindex(imgn).history(hpnt) + $
            'Scaling = [' +  strcompress(arr2str(rts))+']'
   endif     
endfor

return, outdata
end
